program dotNetSpeechApp2;

//You can request individual voice events, or just ask for them all to fire
//This requests them all individually,
//but neither approach causes the AudioLevel event to fire
{$define REQUEST_INDIVIDUAL_EVENTS}

//Shorthand event handler set up for Automation objects gives Internal Error ILCG1474
{$define USE_SHORTHAND_EVENT_SETUP}

uses
  System.Windows.Forms,
  Interop.SpeechLib;

type
  TfrmSAPIRCW = class(Form)
    memWords: TextBox;
    btnSpeak: Button;
    pbVUMeter: ProgressBar;
    procedure btnSpeakClick(Sender: TObject; Args: EventArgs);
    procedure VoiceWord(StreamNumber: Integer; StreamPosition: TObject;
      CharacterPosition, Length: Integer);
    procedure VoiceEndStream(StreamNumber: Integer; StreamPosition: TObject);
    procedure VoiceAudioLevel(StreamNumber: Integer; StreamPosition: TObject; AudioLevel: Integer);
  public
    Voice: SpVoiceClass;//ISpeechVoice;
    constructor Create;
  end;

{ TfrmSAPIRCW }

constructor TfrmSAPIRCW.Create;
begin
  inherited;
  Width := 344;
  Height := 188;
  StartPosition := FormStartPosition.CenterScreen;
  FormBorderStyle := FormBorderStyle.FixedSingle;
  MinimizeBox := False;
  MaximizeBox := False;
  Text := '.NET App Using SAPI Automation Object';
  memWords := TextBox.Create;
  with memWords do
  begin
    Text := 'Hello world!';
    Multiline := True;
    SetBounds(8, 8, 313, 113);
    Parent := Self;
  end;
  btnSpeak := Button.Create;
  with btnSpeak do
  begin
    Text := '&Speak';
    SetBounds(8, 128, 75, 25);
    Parent := Self;
    Add_Click(btnSpeakClick);
  end;
  pbVUMeter := ProgressBar.Create;
  with pbVUMeter do
  begin
    Minimum := 0;
    Maximum := 100;
    Value := 0;
    SetBounds(96, 129, 225, 23);
    Parent := Self;
  end;
  Voice := SpVoiceClass.Create;
{$ifdef USE_SHORTHAND_EVENT_SETUP}
  Voice.Add_Word(VoiceWord);
  Voice.Add_EndStream(VoiceEndStream);
  Voice.Add_AudioLevel(VoiceAudioLevel);
{$else}
  Voice.Add_Word(_ISpeechVoiceEvents_WordEventHandler.Create(
    Self, NativeInt(@TfrmSAPIRCW.VoiceWord)));
  Voice.Add_EndStream(_ISpeechVoiceEvents_EndStreamEventHandler.Create(
    Self, NativeInt(@TfrmSAPIRCW.VoiceEndStream)));
  Voice.Add_AudioLevel(_ISpeechVoiceEvents_AudioLevelEventHandler.Create(
    Self, NativeInt(@TfrmSAPIRCW.VoiceAudioLevel)));
{$endif}
{$ifdef REQUEST_INDIVIDUAL_EVENTS}
  Voice.EventInterests := SpeechVoiceEvents(
    Integer(SpeechVoiceEvents.SVEEndInputStream) +
    Integer(SpeechVoiceEvents.SVEAudioLevel) +
    Integer(SpeechVoiceEvents.SVEWordBoundary));
{$else}
  Voice.EventInterests := SpeechVoiceEvents.SVEAllEvents;
{$endif}
end;

procedure TfrmSAPIRCW.btnSpeakClick(Sender: TObject;
  Args: EventArgs);
begin
  memWords.SelectionLength := 0;
  memWords.Focus;
  Voice.Speak(memWords.Text, SpeechVoiceSpeakFlags.SVSFlagsAsync);
end;

procedure TfrmSAPIRCW.VoiceWord(StreamNumber: Integer;
  StreamPosition: TObject; CharacterPosition, Length: Integer);
begin
  memWords.SelectionStart := CharacterPosition;
  memWords.SelectionLength := Length; //highlight word
end;

procedure TfrmSAPIRCW.VoiceEndStream(StreamNumber: Integer; StreamPosition: TObject);
begin
  //Reset VU meter
  pbVUMeter.Value := 0;
  //Highlight word being spoken in the text box
  memWords.SelectionLength := 0;
  memWords.SelectionStart := Length(memWords.Text);
end;

//This event handler is not fired, despite our requesting it
procedure TfrmSAPIRCW.VoiceAudioLevel(StreamNumber: Integer; StreamPosition: TObject; AudioLevel: Integer);
begin
  pbVUMeter.Value := AudioLevel;
end;

begin
  Application.Run(TfrmSAPIRCW.Create);
end.
